@@ -11,6 +11,7 @@
 #include <arpa/inet.h>
 #include <net/if.h>
 
+#include <fnmatch.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -357,6 +358,41 @@ zeroconf_device_find_by_ident (const char *ident, ID_PROTO *proto)
     return NULL;
 }
 
+/* Check if device is blacklisted
+ *
+ * Returns reason string, if device is blacklisted, NULL, if not
+ */
+static const char*
+zeroconf_device_is_blacklisted (zeroconf_device *device)
+{
+    conf_blacklist *ent;
+    const char     *name, *model;
+
+    if (conf.blacklist == NULL) {
+        return NULL;
+    }
+
+    name = zeroconf_device_name(device);
+    model = zeroconf_device_model(device);
+
+    for (ent = conf.blacklist; ent != NULL; ent = ent->next) {
+        if (ent->name != NULL && !fnmatch(ent->name, name, 0)) {
+            return "name";
+        }
+
+        if (ent->model != NULL && !fnmatch(ent->model, model, 0)) {
+            return "model";
+        }
+
+        if (ent->net.addr.af != AF_UNSPEC &&
+            ip_addrset_on_network(device->addrs, ent->net)) {
+            return "address";
+        }
+    }
+
+    return NULL;
+}
+
 /******************** Merging devices *********************/
 /* Recompute device->buddy for all devices
  */
@@ -1070,7 +1106,7 @@ zeroconf_device_list_get (void)
     for (LL_FOR_EACH(node, &zeroconf_device_list)) {
         zeroconf_device *device;
         ID_PROTO        proto;
-        const char      *name, *model;
+        const char      *name, *model, *blacklisted;
         unsigned int    protocols;
 
         device = OUTER_STRUCT(node, zeroconf_device, node_list);
@@ -1088,6 +1124,14 @@ zeroconf_device_list_get (void)
             continue;
         }
 
+        blacklisted = zeroconf_device_is_blacklisted(device);
+        if (blacklisted != NULL) {
+            log_debug(zeroconf_log,
+                "%s (%d): skipping, device is blacklisted by %s",
+                name, device->devid, blacklisted);
+            continue;
+        }
+
         if (conf.proto_auto && !zeroconf_device_is_mdns(device)) {
             zeroconf_device *device2 = device->buddy;
             if (device2 != NULL && zeroconf_device_protocols(device2) != 0) {
@@ -1354,10 +1398,30 @@ zeroconf_init (void)
 
         for (dev = conf.devices; dev != NULL; dev = dev->next) {
             if (dev->uri != NULL) {
-                log_debug(zeroconf_log, "  %s = %s, %s", dev->name,
+                log_trace(zeroconf_log, "  %s = %s, %s", dev->name,
                     http_uri_str(dev->uri), id_proto_name(dev->proto));
             } else {
-                log_debug(zeroconf_log, "  %s = disable", dev->name);
+                log_trace(zeroconf_log, "  %s = disable", dev->name);
+            }
+        }
+    }
+
+    if (conf.blacklist != NULL) {
+        conf_blacklist *ent;
+
+        log_trace(zeroconf_log, "blacklist:");
+        for (ent = conf.blacklist; ent != NULL; ent = ent->next) {
+            if (ent->model != NULL) {
+                log_trace(zeroconf_log, "  model = %s", ent->model);
+            }
+
+            if (ent->name != NULL) {
+                log_trace(zeroconf_log, "  name = %s", ent->name);
+            }
+
+            if (ent->net.addr.af != AF_UNSPEC) {
+                ip_straddr straddr = ip_network_to_straddr(ent->net);
+                log_trace(zeroconf_log, "  ip = %s", straddr.text);
             }
         }
     }
