@@ -586,6 +586,8 @@ __ptr_array_del (void **a, int i)
 /******************** Safe ctype macros ********************/
 #define safe_isspace(c)         isspace((unsigned char) c)
 #define safe_isxdigit(c)        isxdigit((unsigned char) c)
+#define safe_iscntrl(c)         iscntrl((unsigned char) c)
+#define safe_isprint(c)         isprint((unsigned char) c)
 #define safe_toupper(c)         toupper((unsigned char) c)
 #define safe_tolower(c)         tolower((unsigned char) c)
 
@@ -626,7 +628,7 @@ __ptr_array_del (void **a, int i)
 const char *
 os_homedir (void);
 
-/* Get base name of the calling program. 
+/* Get base name of the calling program.
  * There is no need to free the returned string
  *
  * May return NULL in a case of error
@@ -718,6 +720,28 @@ id_source_sane_name (ID_SOURCE id);
 ID_SOURCE
 id_source_by_sane_name (const char *name);
 
+/* ID_JUSTIFICATION represents hardware-defined ADF justification
+ * This value exposed to the SANE API as a couple of read-only
+ * options, separate for width and height justification.
+ * Not all scanners provide this information
+ */
+typedef enum {
+    ID_JUSTIFICATION_UNKNOWN = -1,
+    ID_JUSTIFICATION_LEFT,
+    ID_JUSTIFICATION_CENTER,
+    ID_JUSTIFICATION_RIGHT,
+    ID_JUSTIFICATION_TOP,
+    ID_JUSTIFICATION_BOTTOM,
+
+    NUM_ID_JUSTIFICATION
+} ID_JUSTIFICATION;
+
+/* id_justification_sane_name returns SANE name for the width justification
+ * For unknown ID returns NULL
+ */
+const char*
+id_justification_sane_name (ID_JUSTIFICATION id);
+
 /* ID_COLORMODE represents color mode
  */
 typedef enum {
@@ -1304,6 +1328,7 @@ struct conf_blacklist {
 typedef struct {
     bool           dbg_enabled;      /* Debugging enabled */
     const char     *dbg_trace;       /* Trace directory */
+    bool           dbg_hexdump;      /* Hexdump all traffic to the trace */
     conf_device    *devices;         /* Manually configured devices */
     bool           discovery;        /* Scanners discovery enabled */
     bool           model_is_netname; /* Use network name instead of model */
@@ -1316,6 +1341,7 @@ typedef struct {
 #define CONF_INIT {                     \
         .dbg_enabled = false,           \
         .dbg_trace = NULL,              \
+        .dbg_hexdump = false,           \
         .devices = NULL,                \
         .discovery = true,              \
         .model_is_netname = true,       \
@@ -1605,6 +1631,9 @@ http_uri_af (http_uri *uri)
 }
 
 /* Get URI path
+ *
+ * Note, if URL has empty path (i.e., "http://1.2.3.4"), the
+ * empty string will be returned
  */
 const char*
 http_uri_get_path (const http_uri *uri);
@@ -2056,6 +2085,12 @@ trace_error (trace *t, error err);
 void
 trace_dump_body (trace *t, http_data *data);
 
+/* Dump binary data (as hex dump)
+ * Each line is prefixed with the `prefix` character
+ */
+void
+trace_hexdump (trace *t, char prefix, const void *data, size_t size);
+
 /******************** SANE_Word/SANE_String arrays ********************/
 /* Create array of SANE_Word
  */
@@ -2433,6 +2468,10 @@ enum {
     OPT_GAMMA,
     OPT_NEGATIVE,
 
+    /* Read-only options for ADF justification */
+    OPT_JUSTIFICATION_X,
+    OPT_JUSTIFICATION_Y,
+
     /* Total count of options, computed by compiler */
     NUM_OPTIONS
 };
@@ -2440,9 +2479,25 @@ enum {
 /* String constants for certain SANE options values
  * (missed from sane/sameopt.h)
  */
-#define OPTVAL_SOURCE_PLATEN      "Flatbed"
-#define OPTVAL_SOURCE_ADF_SIMPLEX "ADF"
-#define OPTVAL_SOURCE_ADF_DUPLEX  "ADF Duplex"
+#define OPTVAL_SOURCE_PLATEN        "Flatbed"
+#define OPTVAL_SOURCE_ADF_SIMPLEX   "ADF"
+#define OPTVAL_SOURCE_ADF_DUPLEX    "ADF Duplex"
+#define OPTVAL_JUSTIFICATION_LEFT   "left"
+#define OPTVAL_JUSTIFICATION_CENTER "center"
+#define OPTVAL_JUSTIFICATION_RIGHT  "right"
+#define OPTVAL_JUSTIFICATION_TOP    "top"
+#define OPTVAL_JUSTIFICATION_BOTTOM "bottom"
+
+/* Define options not included in saneopts.h */
+#define SANE_NAME_ADF_JUSTIFICATION_X  "adf-justification-x"
+#define SANE_TITLE_ADF_JUSTIFICATION_X SANE_I18N("ADF Width Justification")
+#define SANE_DESC_ADF_JUSTIFICATION_X  \
+        SANE_I18N("ADF width justification (left/right/center)")
+
+#define SANE_NAME_ADF_JUSTIFICATION_Y  "adf-justification-y"
+#define SANE_TITLE_ADF_JUSTIFICATION_Y SANE_I18N("ADF Height Justification")
+#define SANE_DESC_ADF_JUSTIFICATION_Y  \
+        SANE_I18N("ADF height justification (top/bottom/center)")
 
 /* Check if option belongs to image enhancement group
  */
@@ -2552,6 +2607,11 @@ typedef struct {
 
     /* Sources */
     devcaps_source *src[NUM_ID_SOURCE];  /* Missed sources are NULL */
+
+    /* ADF Justification */
+    ID_JUSTIFICATION justification_x;   /* Width justification*/
+    ID_JUSTIFICATION justification_y;   /* Height justification*/
+
 } devcaps;
 
 /* Initialize Device Capabilities
@@ -2595,6 +2655,7 @@ typedef struct {
     SANE_Fixed             highlight;         /* 0.0 ... +100.0 */
     SANE_Fixed             gamma;             /* Small positive value */
     bool                   negative;          /* Flip black and white */
+
 } devopt;
 
 /* Initialize device options
